;------------------------------------------------------------------------------
; File Name   :	lprinter.rom
; Base Address:	0000h Range: B800h - C000h Loaded length: 0800h
;
; Druckermodul f. LPRINTER ( vermutlich ein SD11xx, Uni Halle )
;
; reass: Volker Pohlers 26.08.2008, letzte nderung: 26.08.2008
;------------------------------------------------------------------------------

		cpu	Z80
		page	0

;------------------------------------------------------------------------------
; OS
;------------------------------------------------------------------------------

; RAM
IOBYT		equ	0004h		; I/O-Byte zur Zuweisung log.Gert phy.	Gert
LISW		equ	0015h		; Schalter fr Hardcopy. 0 kein Copy, 1 Copy
KEYBU		equ	0025h		; Tastaturpuffer
EOR		equ	0036h		; Zeiger auf letzte fr	Anwender freie Adresse
SYSR		equ	02C0h		; reserviert fr Systemerweiterung
ATTYL		equ	0EFE1h		; Adresse TTY-Treiber fr LIST
TXLST		equ	0EFEFh		; Adresse einer	Zeichenkette des aktuellen LIST-Gertes
BWS		equ	0EC00h		; Bildwiederholspeicher

; CALL5
PRNST		equ 	09h		; Ausgabe einer	Zeichenkette zu	CONST
RCONB		equ 	0Ah		; Eingabe einer	Zeichenkette von CONST


;------------------------------------------------------------------------------
; Druckermodul
;------------------------------------------------------------------------------

; Achtung: SIO andere Adresse als Standard-Modul!
SIODA		EQU	0B1H		; Datenadr SIO Kanal A
SIOKD		EQU	SIODA+2		; Kdoadr   - " -
CTC		EQU	0A8H		; Adr CTC Kanal 0

; Drucker mit Schnittstelle RS232C und DTR-Protokoll (Hardware-Protokoll)

; Baudrate:
;	BDRAT in Bd (50, 75, 100, 150, 200, 300, 600, 1200, 2400, 4800 oder 9600)
; 	Zeitkonstante fuer CTC: ZKCTC=9600/BDRAT
; Datenbit: DABI
;	7 Datenbit: DABI=20H
;	8 Datenbit: DABI=60H
; Paritaetsbit: PABI
;	ohne     Paritaet: PABI=2
;	gerade   Paritaet: PABI=3
;	ungerade Paritaet: PABI=1
; Stopbit: STPBI
;	1   Stopbit: STPBI=4
;	1.5 Stopbit: STPBI=8
;	2   Stopbit: STPBI=0CH

; LPRINTER
ZKCTC		EQU	1		; 9600 Bd
DABI		EQU	20H		; 7 Datenbit
PABI		EQU	1		; ungerade Paritaet
STPBI		EQU	4		; 1 Stopbit

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

		org 0B800h

		jp	eos
		db "#       ",0

		jp	dini
name:		db "LPRINTER",0
		db    0

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

eos:		call	dini
		jp	0F089h		; Sprung ins OS

;Aufbau RAM-Speicher
;Lnge 182 Byte
;IX=(248h)=(EOR)
;
;IX+0
;IX+1,2 akt. Pos. im Zeilenpuffer
;IX+3,4 Beginn Zeilenpuffer
;IX+5,6 Ende Zeilenpuffer
;IX+7,8 Merkzelle fr Zeilenpufferende (z.B. bei Wagenrcklauf)
;IX+9   Merkzelle fr CRLF, FFh -> letzten Zeichen war CR, sonst 0
;IX+10  Endlospapier ('J'/'N') Bit2=0 -> 'J'
;IX+11  Zeilen pro Seite
;IX+12  akt. Anz. Zeilen bis Seitenende
;IX+13  Leerzeilen pro Seite
;IX+14  Zeichen pro Zeile
;IX+15  Eingabepuffer, max Lnge
;IX+16		aktuelle Lnge
;IX+17..20	1..4 Zeichen
;IX+20..181 Zeilenpuffer (162 Zeichen)

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------

dini:		ld	hl, (TXLST)	; Zeichenkette f. LIST eintragen
		ld	de, name	; "LPRINTER"
		and	a		; Cy = 0
		sbc	hl, de
		push	af
		ld	(TXLST), de
		push	de
		jr	z, ini1		; Druckertreiber schon init.
;
		ld	hl, 2FFh
		ld	de, 182
		and	a
		sbc	hl, de
		nop
		nop
		nop
		ld	(248h),	hl
		push	hl
		pop	ix
		ld	de, 20
		add	hl, de
		ld	(ix+4),	h	; Beginn Zeilenpuffer
		ld	(ix+3),	l
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ld	de, 132
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l

		ld	(ix+9),	0	; CR-Merker rcksetzen
;Programmierung CTC und SIO
		ld	hl, PROGT
		ld	c, CTC		; Prog. CTC
		ld	b, 2
		otir
		ld	c, SIOKD	; Prog. SIO
		ld	b, 8
		otir
;
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
;
ini1:		ld	hl, ttyl	; TTY-Treiber f. LIST
		ld	(ATTYL), hl
		ld	hl, 4		; LIST := ATTYP
		res	7, (hl)
		res	6, (hl)
		xor	a
		ld	(LISW), a	; kein Hardcopy
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		pop	de
		call	5		; Ausg.	"LPRINTER"
		pop	af
		call	nz, inidlg
		ld	b, 1
		call	stat1		; kurze	Statusabfrage
		ld	de, aNichtBereit ; " nicht bereit ! "
		ld	c, PRNST	; Ausgabe einer	Zeichenkette zu	CONST
		call	c, 5

diniret:	and	a		; Cy = 0, d.h. kein Fehler
		ret


;------------------------------------------------------------------------------
; Initialisierungstabelle CTC und SIO
;------------------------------------------------------------------------------

PROGT:
;CTC
		db 00010111b		; Zeitgeber/DI/Vorteiler 16
		db ZKCTC		; Zeitkonstante
;SIO
		db 4			; Steuerregister
		db 40H+PABI+STPBI	; keine	Paritt,1 Stoppbit,Vorteiler 16,Monosynch-Mode
		db 1			; Interruptvektor
		db 0			; Kein Interrupt
		db 3			; Steuerung Empfnger
		db 01000001b		;
		db 5			; Steuerung Sender
;00101000b
		db DABI+8		; 8 bit, Sender	ein

aNichtBereit:	db " nicht bereit ! ",0

;------------------------------------------------------------------------------
; Einrichten des logischen Treibers
; (Bedienerdialog fr Papierformat etc.)
;------------------------------------------------------------------------------

ixinit:
		ld	(ix+10), 'J'	; Endlospapier
		ld	(ix+11), 60	; Zeilen pro Seite
		ld	(ix+12), 60	; akt. Anz. Zeilen bis Seitenende
		ld	(ix+13), 12	; Leerzeilen pro Seite
		ld	(ix+14), 80	; Zeichen pro Zeile
		ret

inidlg:		ld	a, 7Fh ; ''
		out	(0B1h),	a
		ld	b, 30h ; '0'
		call	stat1
		jr	nc, loc_B8E1
		ld	de, aNichtBereit ; " nicht bereit ! "
		ld	c, 9
		call	5
		jr	loc_B8E9

loc_B8E1:
		ld	de, aPapierPosition ; "\r\nPapier positionieren	!\r\n"
		ld	c, 9
		call	5

loc_B8E9:
		call	ixinit
		ld	hl, diniret
		ex	(sp), hl
		ret

aPapierPosition:db 0Dh,0Ah
		db "Papier positionieren !",0Dh,0Ah,0

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; TTY-Treiber f. LIST (logischer Treiber)
;
; Treibereingangsparameter:
; - A Kommando (siehe unten)
; - C Zeichen (bei Kommando AUSGABE)
;
; Treiberausgangsparameter:
; - CY Fehlerstatus
;
; Kommando
; 00 Abfrage Status
; 	Return:
; 	A Status
; 	0 kein Zeichen bei Eingabegert, nicht bereit bei Ausgabegert
; 	sonst Zeichen liegt an bei Eingabegert,
; 	(im installierten CRT-Treiber wird der Zeichencode bergeben)
; 02 Ausgabe Zeichen
; 	Eingang:
; 	C Zeichen
; (FF Initialisieren/Rcksetzen des Gertes)
;
; Es wird nur 00 ausgewertet, alle anderen codes werden ignoriert -> Zeichenausgabe
; FF ist nicht implementiert
;------------------------------------------------------------------------------

ttyl:		and	a		; TTY-Treiber f. LIST
		ld	b, 1
		jp	z, stat1	; wenn A=0 -> Statusabfrage
		push	ix
		ld	ix, (248h)
		ld	hl, ttylret
		push	hl		; RETURN-Adresse
		ld	a, c		; A=auszugebendes Zeichen
		ld	hl, ttyltab1	; Tabelle der Steuerzeichen
		ld	bc, 7
		cpir			; Steuerzeichen	suchen
		jr	z, ttyl1	; wenn A Steuerzeichen
		dec	bc		; bc :=	-1, wenn kein Steuerz., so Funktion char_xx
;
ttyl1:		ld	hl, 6
		and	a
		sbc	hl, bc
		add	hl, hl
		ld	bc, ttyltab2	; Tabelle der Steuerzeichen-Funktionen
		add	hl, bc
		ld	c, (hl)
		inc	hl
		ld	h, (hl)
		ld	l, c
		jp	(hl)		; Steuerzeichenfunktion	starten
;
ttylret:	pop	ix
		ret	nc
		ld	a, 0
		ld	(LISW), a	; kein Hardcopy
		ret

; Tabelle der Steuerzeichen
ttyltab1:	db 0Dh
		db 0Ah
		db 0Ch
		db 9
		db 17h
		db 0Eh
		db 8
; Tabelle der Steuerzeichen-Funktionen
ttyltab2:	dw char_0d		; Wagenrcklauf CR
		dw char_0a		; Cursor down
		dw char_0c		; CRLF (CLS?)
		dw char_09		; Cursor right
		dw char_17		; CTRL/W Seitenvorschub	am Drucker
		dw char_0e		; CTRL/N Bildschirmkopie
		dw char_08		; Cursor left
		dw char_xx		; alle anderen Zeichen


;------------------------------------------------------------------------------
; Steuerzeichenfunktionen
;------------------------------------------------------------------------------

; Wagenrcklauf CR
char_0d:	ld	(ix+9),	0FFh	; CR merken
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		and	a		; Cy := 0
		ret

; Cursor down LF
char_0a:	bit	0, (ix+9)	; war CR?
		jr	nz, char_0a_1	; wenn ja
;
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(ix+8),	h	; merken
		ld	(ix+7),	l
char_0a_1:	ld	(ix+9),	0	; CR-Merker rcksetzen
		call	outzp		; Zeilenpuffer ausgeben
		ret

; CLS -> CRLF
char_0c:	call	char_0d		; CR
		call	char_0a		; LF
		ret

; Cursor right
char_09:	ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	d, (ix+6)	; Ende Zeilenpuffer
		ld	e, (ix+5)
		and	a		; Cy := 0
		sbc	hl, de		; Zeilenende erreicht?
		ccf
		call	z, char_0c	; ja -> CRLF
		ld	(ix+9),	0	; CR-Merker rcksetzen
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		inc	hl
		ld	(ix+2),	h	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	l
		ret

; CTRL/W Seitenvorschub	am Drucker
char_17:	bit	2, (ix+10)	; Endlospapier?
		jr	nz, char_17_1	; nein
;
		ld	a, (ix+13)	; Leerzeilen pro Seite
		and	a		; <> 0 ?
		jr	nz, char_17_1
;
		call	char_0c		; CRLF
		ret
;wenn Endlospapier
char_17_1:	ld	b, (ix+12)	; akt. Anz. Zeilen bis Seitenende
char_17_2:	push	bc
		call	char_0c		; CRLF
		pop	bc
		ret	c
		djnz	char_17_2
		ret

; sonstige Zeichen
char_xx:	cp	20h
		jr	nc, char_xx_1	; Steuerzeichen < 20h bergehen
		and	a		; Cy := 0
		ret
;
char_xx_1:	push	af
		call	char_09		; Cursor right
		pop	de
		ld	h, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	l, (ix+1)
		ld	(hl), d		; Zeichen im Zeilenpuffer eintragen
		ret


; CTRL/N Bildschirmkopie
char_0e:	call	char_0c		; CRLF
		ret	c
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 40
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 40 Zeichen/Zeile verkleinern
;
		ld	de, BWS		; Bildwiederholspeicher
char_0e_1:	ld	a, (de)
		push	de
		call	char_xx		; Zeichen ausgeben (logisch)
					; Zeilenwechsel erfolgt automatisch
		pop	de
		ret	c
		inc	de
		ld	hl, BWS+40*24-1	; BWS-Ende
		and	a
		sbc	hl, de
		jr	nc, char_0e_1
;
		call	char_0c		; CRLF
		ret	c
;
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	de, 132
		add	hl, de
		ld	(ix+6),	h	; Ende Zeilenpuffer
		ld	(ix+5),	l	; auf 132 Zeichen/Zeile restaurieren
		ret

; Cursor left
char_08:	ld	(ix+9),	0	; CR-Merker rcksetzen
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	d, (ix+2)	; akt. Pos. im Zeilenpuffer
		ld	e, (ix+1)
		and	a
		sbc	hl, de		; Am Anfang der Zeile?
		ret	z		; dann gibt es nicht zu tun
		dec	de		; sonst 1 Zeichen nach links
		ld	(ix+2),	d	; akt. Pos. im Zeilenpuffer
		ld	(ix+1),	e
		and	a
		ret


;------------------------------------------------------------------------------
; Zeilenpuffer-Routinen
;------------------------------------------------------------------------------

; Zeilenpuffer mit Leerzeichen init.
clrzp:		push	af
		ld	h, (ix+4)	; Beginn Zeilenpuffer
		ld	l, (ix+3)
		ld	bc, 162		; Lnge Zeilenpuffer
clrzp1:		inc	hl
		ld	(hl), ' '
		dec	bc
		ld	a, b
		or	c
		jr	nz, clrzp1
		pop	af
		ret

; Zeilenpuffer ausgeben
outzp:		call	prepzp		; Zeilenpuffer fr Ausdrucken vorbereiten
		call	prnzp		; Ausdrucken einer Zeile
		call	clrzp		; Zeilenpuffer mit Leerzeichen init.
		ret

; Zeilenpuffer fr Ausdrucken vorbereiten
prepzp:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		push	hl
		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
		and	a
		sbc	hl, bc
		ld	b, h		; BC := HL := Anzahl auszugebender Zeichen
		ld	c, l
		ex	(sp), hl	; HL := Zeilenpufferende
		push	hl
		jr	z, prepzp1	; wenn keine Zeichen auszugeben
; sonst Zeilenpuffer 1 Zeichen nach rechts verschieben
		ld	d, h
		ld	e, l
		inc	de
		lddr
;
prepzp1:	inc	hl
		ld	(hl), 0Dh	; als erstes Zeichen CR eintragen
;
		pop	hl		; HL := Zeilenpufferende
prepzp2:	inc	hl
		ex	(sp), hl
		ld	b, 0
		ld	c, (ix+14)	; Zeichen pro Zeile
		and	a
		sbc	hl, bc
		ld	b, h
		ld	c, l
		ex	(sp), hl
		jr	z, prepzp3
		jr	c, prepzp3
; wenn Zeilenlnge erreicht, so LFCR einfgen
		push	hl
		ld	d, h
		ld	e, l
		inc	de
		inc	de
		lddr
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		inc	hl
		ld	(hl), 0Dh	; CR einfgen
		pop	hl
		inc	hl
		jr	prepzp2
;
prepzp3:	pop	de
		inc	hl
		ld	(hl), 0Ah	; LF einfgen
		ld	(ix+8),	h	; Merkzelle fr Zeilenpufferende
		ld	(ix+7),	l
		ret

; Ausdrucken einer Zeile
prnzp:		ld	b, (ix+4)	; Beginn Zeilenpuffer
		ld	c, (ix+3)
prnzp1:		ld	h, (ix+8)	; Merkzelle fr Zeilenpufferende
		ld	l, (ix+7)
		and	a
		sbc	hl, bc
		ret	z		; wenn kein Zeichen auszugeben ist
;
prnzp2:		inc	bc
		ld	a, (bc)
		push	bc
		call	prpgr		; Druckzeichen aus A nach D kopieren,
					; Grafikzeichen	als '*'
		call	dout		; Ausgabe D
		pop	bc
		ret	c
		ld	a, (bc)
		cp	0Ah
		jr	nz, prnzp2	; Zeichen ausgeben bis LF
;
		call	newpg		; dann evtl. Seitenwechsel
		jr	prnzp1		; und restl. Zeichen ausgeben

; Seitenwechsel
newpg:		dec	(ix+12)		; akt. Anz. Zeilen bis Seitenende
		ret	nz
;
		ld	a, (ix+11)	; Zeilen pro Seite
		ld	(ix+12), a	; akt. Anz. Zeilen bis Seitenende
		bit	2, (ix+10)	; Endlospapier?
		jr	z, newpg2	; nein
;
		ld	hl, KEYBU	; Warten auf Tastendruck
newpg1:		ld	a, (hl)
		and	a
		jr	z, newpg1
		ld	(hl), 0
		ret
; Endlospapier --> Ausgabe der Leerzeilen
newpg2:		ld	e, (ix+13)	; Leerzeilen pro Seite
		ld	a, e
		and	a
		ret	z
newpg3:		ld	d, 0Ah
		push	bc
		call	dout		; Ausgabe D
		pop	bc
		jr	c, newpgret	; bei Fehler
		dec	e
		jr	nz, newpg3
		ret


; Druckzeichen aus A nach D kopieren,
; Grafikzeichen	als '*'
prpgr:		ld	d, a
		cp	7Fh
		ret	c
		ld	d, '*'
		ret

;
newpgret:	pop	bc
		ret

;------------------------------------------------------------------------------
; phys. Treiber
;------------------------------------------------------------------------------

; Ausgabe D
dout:		call	stat		; lange	Statusabfrage
		ret	c
		ld	a, d
		out	(SIODA), a
		and	a
		ret

; Statusabfrage
status:		in	a, (SIODA)
		nop
		nop
		nop
		and	7Fh
		xor	11h
		ret	z
		scf
		ret

; lange	Statusabfrage
stat:		ld	b, 200		; Abfrage Status

; kurze	Statusabfrage
stat1:		ld	c, 200
stat2:		call	wait		; Zeitschleife
		call	status		; Statusabfrage
		ret	nc
		dec	c
		jr	nz, stat2
		djnz	stat1		; kurze	Statusabfrage
		scf
		ret

; Zeitschleife
wait:		push	bc
		ld	bc, 187h
wait1:		dec	c
		nop
		jr	nz, wait1
		djnz	wait1
		pop	bc
		ret


		end
